import { concat, mapObject } from "../../pseuco-shared-components/util/functional";
import LTSSVGFragmentView, { XY, stateCircleSize } from "../../pseuco-shared-components/ui/editors/lts/LTSSVGFragmentView";
import { LTSViewData } from "../../pseuco-shared-components/ui/editors/lts/LTSInteractiveView";
import React, { Fragment } from "react";

export type Side = "left" | "right";
export const sides: Side[] = ["left", "right"];
export const isSide = (input: unknown): input is Side => input === "left" || input === "right";

const scale = 1;

export const otherSide = (side: Side): Side => side === "left" ? "right" : "left";

export type DoubleSided<T> = { [side in Side]: T };

/**
 * A pair of LTS states.
 */
export type Pair = DoubleSided<string>;

export const shiftToTransform = (shift: XY): string => `translate(${shift.x}px, ${shift.y}px)`;

export const addShifts = (...shifts: XY[]): XY => ({ x: shifts.map((s) => s.x).reduce((a, b) => a + b), y: shifts.map((s) => s.y).reduce((a, b) => a + b) });

export const shiftToCoords = (shift: XY): string => `${shift.x} ${shift.y}`;

export const getMaxLTSCoordinate = (lts: LTSViewData): XY => {
    const allThings = concat<XY>(Object.values(lts.states), lts.transitions);
    const x = allThings.map((n) => n.x).reduce((a, b) => Math.max(a, b)) + stateCircleSize(scale);
    const y = allThings.map((n) => n.y).reduce((a, b) => Math.max(a, b)) + stateCircleSize(scale);
    return { x, y };
};

export const prepareDoubleLTSForRendering = (lts: DoubleSided<LTSViewData>, margins: {
    top: number;
    bottom: number;
    left: number;
    right: number;
}, innerMargin: number): {
    shifts: DoubleSided<XY>;
    imageSize: XY;
    doubleLtsElement: React.ReactElement;
} => {
    // measure LTS sizes
    const measurements = mapObject(lts, getMaxLTSCoordinate);

    const avgHeight = (measurements.left.y + measurements.right.y) / 2;

    const shifts: DoubleSided<XY> = {
        left: {
            x: margins.left,
            y: margins.top + Math.max(0, avgHeight - measurements.left.y)
        },
        right: {
            x: margins.left + measurements.left.x + innerMargin,
            y: margins.top + Math.max(0, avgHeight - measurements.right.y)
        }
    };

    const imageSize: XY = {
        x: margins.left + measurements.left.x + innerMargin + measurements.right.x + margins.right,
        y: margins.top + Math.max(measurements.left.y, measurements.right.y) + margins.bottom
    };

    return {
        shifts,
        imageSize,
        doubleLtsElement: <Fragment>
            <g style={{ transform: shiftToTransform(shifts.left)}}>
                <LTSSVGFragmentView states={lts.left.states} transitions={lts.left.transitions} showExpandNotice={false} />
            </g>
            <g style={{ transform: shiftToTransform(shifts.right)}}>
                <LTSSVGFragmentView states={lts.right.states} transitions={lts.right.transitions} showExpandNotice={false} />
            </g>
        </Fragment>
    };
};
