import { mapObject } from "../../pseuco-shared-components/util/functional";
import { LTSViewData } from "../../pseuco-shared-components/ui/editors/lts/LTSInteractiveView";

export type LTSTransitionViewDataAlike = {
    source: string;
    weak?: boolean;
    label: string;
    target: string;
};


export const generateWeakTransitions = (lts: LTSViewData): LTSTransitionViewDataAlike[] => {
    const res: LTSTransitionViewDataAlike[] = [];

    // step 1: find all connections made by τ-transitions

    const tauConnections: { [stateKey: string]: string[] } = mapObject(lts.states, (state, stateKey) => [stateKey]);
    lts.transitions.forEach((transition) => {
        if (transition.weak) {
            if (!tauConnections[transition.source].includes(transition.target)) tauConnections[transition.source].push(transition.target);
        }
    });
    // build transitive closure
    while (true) {
        let unchanged = true;

        mapObject(tauConnections, (s2Connections, s2) => s2Connections.forEach((s3) => {
            // we know s2 ⇒ s3, look for s1 with s1 ⇒ s2
            mapObject(tauConnections, (s1Connections) => {
                if (s1Connections.includes(s2) && !s1Connections.includes(s3)) {
                    s1Connections.push(s3);
                    unchanged = false;
                }
            });
        }));

        if (unchanged) break;
    }

    // step 2½: insert all τ-connections as weak transitions
    mapObject(tauConnections, (connections, s) => connections.forEach((t) => res.push({
        source: s,
        target: t,
        weak: true,
        label: "τ"
    })));

    // step 3: for non-τ transitions, combine a tau connection, a real step and another tau connection in every possible way
    lts.transitions.forEach(({ source, target, weak, label }) => {
        if (weak) return;

        const predecessors = Object.keys(tauConnections).filter((s) => tauConnections[s].includes(source));
        const successors = tauConnections[target];

        predecessors.forEach((predecessor) => successors.forEach((successor) => {
            if (res.some((t) => t.source === predecessor && t.target === successor && t.label === label)) return; // suppress duplicate
            res.push({
                source: predecessor,
                target: successor,
                label,
                weak: false
            });
        }));
    });

    return res;
};