import { flowReachabilityGraphValidator, FlowReachabilityGraphValidatorConfiguration } from "../../flowValidation";
import { moveValidatorAnd, moveValidatorHandshaking, moveValidatorMessagePassing, moveValidatorParticipatingAgents, moveValidatorPrintLn, moveValidatorStartAgent } from "../../moveValidators";
import { registerValidationWorkerCallback } from "../../validation";

const validatorConfiguration: FlowReachabilityGraphValidatorConfiguration = {
    steps: {
        order: "sequential",
        steps: [
            {
                moveCompletesStep: moveValidatorStartAgent(),
                description: `start an agent`
            },
            {
                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([0, 1]), moveValidatorHandshaking((v) => /^World!?$/i.test(v.toString()), false)),
                description: `send "World" from the main agent to the first started agent on a synchronous channel`
            },
            {
                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([1]), moveValidatorPrintLn(/^Hello,? World!?$/i, false)),
                description: `print "Hello, World!" (from the first started agent)`
            }
        ]
    },
    restrictedMoves: [
        {
            detector: moveValidatorMessagePassing(),
            description: "performed a message-passing action"
        }
    ]
};

registerValidationWorkerCallback(flowReachabilityGraphValidator(validatorConfiguration), { mp: true, sm: false });
