import { flowReachabilityGraphValidator, FlowReachabilityGraphValidatorConfiguration } from "../../flowValidation";
import { moveValidatorAnd, moveValidatorAsyncChannel, moveValidatorMessagePassing, moveValidatorParticipatingAgents, moveValidatorPrintLn, moveValidatorStartAgent } from "../../moveValidators";
import { registerValidationWorkerCallback } from "../../validation";

const validatorConfiguration: FlowReachabilityGraphValidatorConfiguration = {
    steps: {
        order: "sequential",
        steps: [
            {
                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([0]), moveValidatorAsyncChannel("send", (v) => v === "Hello", false)),
                description: `send "Hello" on an asynchronous channel from the main agent`
            },
            {
                moveCompletesStep: moveValidatorStartAgent(),
                description: `start an agent`
            },
            {
                order: "parallel",
                steps: [
                    {
                        moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([0]), moveValidatorAsyncChannel("send", (v) => v === "World", false)),
                        description: `send "World" on an asynchronous channel from the main agent`
                    },
                    {
                        moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([1]), moveValidatorAsyncChannel("receive", (v) => v === "Hello", false)),
                        description: `receive "Hello" on an asynchronous channel from the first started agent`
                    }
                ]
            },
            {
                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([1]), moveValidatorAsyncChannel("receive", (v) => v === "World", false)),
                description: `receive "World" on an asynchronous channel from the first started agent`
            },
            {
                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([1]), moveValidatorPrintLn(/^Hello,? World!?$/, false)),
                description: `print "Hello, World!" (from the first started agent)`
            }
        ]
    },
    restrictedMoves: [
        {
            detector: moveValidatorMessagePassing(),
            description: "performed a message-passing action"
        }
    ]
};

registerValidationWorkerCallback(flowReachabilityGraphValidator(validatorConfiguration), { mp: true, sm: false });
