import { FlowElement, flowReachabilityGraphValidator, FlowReachabilityGraphValidatorConfiguration } from "../../flowValidation";
import { moveValidatorAnd, moveValidatorHandshaking, moveValidatorMessagePassing, moveValidatorParticipatingAgents, moveValidatorPrintLn } from "../../moveValidators";
import { registerValidationWorkerCallback } from "../../validation";

const printAgentPrintStep = (prime: number): FlowElement => ({
    moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([4]), moveValidatorPrintLn(new RegExp(`^${prime}$`), false)),
    description: `agent 4: print ${prime}`
});

const agentIdToDescription = (agent: number) => agent >= 0 ? `agent ${agent}` : `mainAgent`;

const handshakeStep = (sender: number, receiver: number) => (value: number): FlowElement => ({
    moveCompletesStep: moveValidatorHandshaking((v) => v == value, true, { sender, receiver, allowOtherAgents: true }),
    description: `send ${value} from ${agentIdToDescription(sender)} to ${agentIdToDescription(receiver)}`
});

const validatorConfiguration: FlowReachabilityGraphValidatorConfiguration = {
    steps: {
        order: "parallel",
        steps: [
            {
                order: "sequential",
                steps: [6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18, 19, 20, 21, 22, 23, 24, 25].map(handshakeStep(0, 1))
            },
            {
                order: "sequential",
                steps: [7, 9, 11, 13, 15, 17, 19, 21, 23, 25].map(handshakeStep(1, 2))
            },
            {
                order: "sequential",
                steps: [7, 11, 13, 17, 19, 23, 25].map(handshakeStep(2, 3))
            },
            {
                order: "sequential",
                steps: [7, 11, 13, 17, 19, 23].map(handshakeStep(3, 4))
            },
            {
                order: "sequential",
                steps: [7, 11, 13, 17, 19, 23].map(printAgentPrintStep)
            }
        ]
    },
    restrictedMoves: [
        {
            detector: moveValidatorMessagePassing(),
            description: "performed a message-passing action"
        }
    ]
};

registerValidationWorkerCallback(flowReachabilityGraphValidator(validatorConfiguration), { mp: true, sm: false }, [], 100);
