import { flowReachabilityGraphValidator, FlowReachabilityGraphValidatorConfiguration } from "../../flowValidation";
import { moveValidatorAnd, moveValidatorAsyncChannel, moveValidatorMessagePassing, moveValidatorParticipatingAgents, moveValidatorPrintLn, moveValidatorStartAgent, moveValidatorTerminateAgent } from "../../moveValidators";
import { registerValidationWorkerCallback } from "../../validation";

const validatorConfiguration: FlowReachabilityGraphValidatorConfiguration = {
    steps: {
        order: "sequential",
        steps: [
            {
                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([0, 1]), moveValidatorStartAgent()),
                description: "start an agent acting as the primer()"
            },
            {
                order: "parallel",
                steps: [
                    {
                        order: "sequential", // primer()
                        steps: [
                            {
                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([1]), moveValidatorAsyncChannel("send", (v) => v == 2, false)),
                                description: "agent 1: send 2 to an asynchronous channel"
                            },
                            {
                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([1]), moveValidatorAsyncChannel("send", (v) => v == 3, false)),
                                description: "agent 1: send 3 to an asynchronous channel"
                            },
                            {
                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([1]), moveValidatorAsyncChannel("send", (v) => v == 5, false)),
                                description: "agent 1: send 5 to an asynchronous channel"
                            },
                            {
                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([1]), moveValidatorTerminateAgent()),
                                description: "terminate agent 1"
                            }
                        ]
                    },
                    {
                        order: "sequential", // mainAgent
                        steps: [
                            {
                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([0, 2]), moveValidatorStartAgent()),
                                description: "start an agent acting as the factorial() agent"
                            },
                            {
                                order: "parallel",
                                steps: [
                                    {
                                        order: "sequential", // factorial()
                                        steps: [
                                            {
                                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([2]), moveValidatorAsyncChannel("receive", (v) => v == 2, false)),
                                                description: "agent 2: receive 2 from an asynchronous channel"
                                            },
                                            {
                                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([2]), moveValidatorPrintLn(/^2$/, false)),
                                                description: "agent 2: print 2!"
                                            },
                                            {
                                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([2]), moveValidatorAsyncChannel("receive", (v) => v == 3, false)),
                                                description: "agent 2: receive 3 from an asynchronous channel"
                                            },
                                            {
                                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([2]), moveValidatorPrintLn(/^6$/, false)),
                                                description: "agent 2: print 3!"
                                            },
                                            {
                                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([2]), moveValidatorAsyncChannel("receive", (v) => v == 5, false)),
                                                description: "agent 2: receive 5 from an asynchronous channel"
                                            },
                                            {
                                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([2]), moveValidatorPrintLn(/^120$/, false)),
                                                description: "agent 2: print 5!"
                                            }
                                        ]
                                    },
                                    {
                                        moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([0]), moveValidatorTerminateAgent()),
                                        description: "terminate the mainAgent"
                                    }
                                ]
                            }
                        ]
                    }
                ]
            }
        ]
    },
    restrictedMoves: [
        {
            detector: moveValidatorMessagePassing(),
            description: "performed a message-passing action"
        },
        {
            detector: moveValidatorStartAgent(),
            description: "started an agent"
        },
        {
            detector: moveValidatorTerminateAgent(),
            description: "terminated an agent"
        }
    ]
};

registerValidationWorkerCallback(flowReachabilityGraphValidator(validatorConfiguration), { mp: true, sm: false });
