import { flowReachabilityGraphValidator, FlowReachabilityGraphValidatorConfiguration } from "../../flowValidation";
import { moveValidatorAnd, moveValidatorHandshaking, moveValidatorMessagePassing, moveValidatorParticipatingAgents, moveValidatorPrintLn, moveValidatorSelectCase, moveValidatorStartAgent, moveValidatorTerminateAgent } from "../../moveValidators";
import { registerValidationWorkerCallback } from "../../validation";

const validatorConfiguration: FlowReachabilityGraphValidatorConfiguration = {
    steps: {
        order: "sequential",
        steps: [
            {
                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([0, 1]), moveValidatorStartAgent()),
                description: "start an agent"
            },
            {
                moveCompletesStep: moveValidatorHandshaking((v) => v === 3, false, { sender: 0, receiver: 1, allowOtherAgents: false }),
                description: "send the number 3 to the factorial agent (which should be in a select-case)"
            },
            {
                moveCompletesStep: moveValidatorHandshaking((v) => v === 6, false, { sender: 1, receiver: 0, allowOtherAgents: false }),
                description: "send 3! from the factorial agent to the main agent"
            },
            {
                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([0]), moveValidatorPrintLn(/^3! evaluates to 6\.$/, false)),
                description: `print "3! evaluates to 6." from the main agent`
            },
            {
                moveCompletesStep: moveValidatorAnd(moveValidatorHandshaking((v) => v === 6, false, { sender: 0, receiver: 1, allowOtherAgents: false }), moveValidatorSelectCase("handshake-receive", 0, 2, 0, false)),
                description: "send 3! to the factorial agent (which should be in a select-case)"
            },
            {
                moveCompletesStep: moveValidatorHandshaking((v) => v === 720, false, { sender: 1, receiver: 0, allowOtherAgents: false }),
                description: "send 3!! from the factorial agent to the main agent"
            },
            {
                order: "parallel",
                steps: [
                    {
                        moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([0]), moveValidatorPrintLn(/^\(3!\)! evaluates to 720\.$/, false)),
                        description: `print "(3!)! evaluates to 720." from the main agent`
                    },
                    {
                        order: "sequential",
                        steps: [
                            {
                                moveCompletesStep: moveValidatorAnd(moveValidatorHandshaking((v) => typeof v === "boolean", false, { sender: 0, receiver: 1, allowOtherAgents: false }), moveValidatorSelectCase("handshake-receive", 0, 2, 0, false)),
                                description: "send a boolean to the factorial agent (which should be in a select-case)"
                            },
                            {
                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([1]), moveValidatorTerminateAgent()),
                                description: "terminate the factorial agent"
                            }
                        ]
                    }
                ]
            }
        ]
    },
    restrictedMoves: [
        {
            detector: moveValidatorMessagePassing(),
            description: "performed a message-passing action"
        }
    ]
};

registerValidationWorkerCallback(flowReachabilityGraphValidator(validatorConfiguration), { mp: true, sm: false });
