
import { flowReachabilityGraphValidator, FlowReachabilityGraphValidatorConfiguration } from "../../flowValidation";
import { dropMoveValidatorResultValue, moveValidatorAnd, moveValidatorCallProcedure, moveValidatorDetectAndRequire, moveValidatorNot, moveValidatorParticipatingAgents, moveValidatorPrintLn, moveValidatorRecursionDepth, moveValidatorStartAgent } from "../../moveValidators";
import { registerValidationWorkerCallback } from "../../validation";

const validatorConfiguration: FlowReachabilityGraphValidatorConfiguration = {
    steps: {
        order: "sequential",
        steps: [
            {
                moveCompletesStep: moveValidatorCallProcedure((args) => {
                    const values = Object.values(args);
                    if (values.length !== 1) return { success: false, error: `A procedure was called with ${values.length} arguments, but one argument was expected.` };
                    const value = values[0];
                    if (typeof value === "string" && /^(World|world)!?$/.test(value)) {
                        return { success: true, value: true };
                    } else {
                        return { success: false, error: `A procedure was called with the argument ${JSON.stringify(value)}, but "World" was expected.` };
                    }
                }),
                description: `call a procedure and pass the argument "World"`
            },
            {
                moveCompletesStep: moveValidatorDetectAndRequire(moveValidatorPrintLn(/^(Hello|hello),? (World|world)!?$/, false), moveValidatorAnd(moveValidatorParticipatingAgents([0]), moveValidatorRecursionDepth(1)), `Expected "Hello World!" to be printed from a procedure called by the main agent.`),
                description: `print "Hello, World!" from the procedure`
            }
        ]
    },
    isLegalMove: dropMoveValidatorResultValue(moveValidatorNot(moveValidatorStartAgent(), "In this exercise, you may not start agents."))
};

registerValidationWorkerCallback(flowReachabilityGraphValidator(validatorConfiguration), { mp: false, sm: false });
