
import { moveValidatorDisallowDataRaces } from "../../moveSetValidators";
import { syntaxValidatorDisallowGlobalVariables } from "../../syntaxValidation";
import { ReachabilityGraphValidator, registerValidationWorkerCallback } from "../../validation";

// We cannot use the regular flow reachability validator here – we're trying to ensure a specific kind of move set exists anywhere in the graph, which does not fit the usual structure.

let foundDataRace = false;

const validator: ReachabilityGraphValidator<null> = {
    initialState: null,
    acceptMove: () => ({ success: true, value: null }),
    cycleAllowed: () => ({ success: true, value: null }),
    deadlockAllowed: () => ({ success: true, value: null }),
    moveSetAllowed: (_null, moves) => {
        // we allow any move set during search, but want to take note whether we saw a data race
        if (!foundDataRace) {
            const dataRaceDetectorResult = moveValidatorDisallowDataRaces()(moves);
            if (dataRaceDetectorResult.success === false) foundDataRace = true;
        }
        return { success: true, value: null };
    },
    endSearchAllowed: () => {
        if (foundDataRace) {
            return { success: true, value: null };
        } else {
            return { success: false, error: "Your program does not have a data race." };
        }
    }
};

registerValidationWorkerCallback(validator, { mp: false, sm: true }, [syntaxValidatorDisallowGlobalVariables()]);
