import React from "react";
import { PseuCoProgrammingConfiguration } from "../../pseuCoProgramming";

const config: PseuCoProgrammingConfiguration = {
    allowedDialects: {
        mp: false,
        sm: true
    },
    exerciseDescription: <div>
        <p>
            Add proper use of conditions to this program to ensure the following:
        </p>
        <ul>
            <li>
                <code>putMessage()</code> should wait for the <code>MessageBox</code> to be empty (instead of overwriting any already-stored value).
            </li>
            <li>
                <code>getMessage()</code> should wait for a message to be available (instead of returning -1 or a previous message).
            </li>
        </ul>
        <p>
            Make no other changes.
        </p>
    </div>,
    template: `monitor MessageBox {
    int message = -1;
    bool hasMessage = false;
    
    void putMessage(int msg) {
        message = msg;
        hasMessage = true;
    }
    
    int getMessage() {
        hasMessage = false;
        return message;
    }
}

void sender(MessageBox m) {
    m.putMessage(42);
    m.putMessage(1337);
}

void printer(MessageBox m) {
    while (true) {
        println(m.getMessage());
    };
}

mainAgent {
    MessageBox m;
    start(printer(m));
    start(sender(m));
}`,
    getWorker: () => new Worker(new URL('./worker.ts', import.meta.url))
};

export default config;
