import { VariableDeclarations, SimpleTypeNode, Struct } from "@pseuco/lang";
import { flowReachabilityGraphValidator, FlowReachabilityGraphValidatorConfiguration } from "../../flowValidation";
import { moveValidatorDisallowDataRaces } from "../../moveSetValidators";
import { moveValidatorAnd, moveValidatorParticipatingAgents, moveValidatorPrintLn, moveValidatorReadPseuCoStruct, moveValidatorStartAgent, moveValidatorWritePseuCoStruct } from "../../moveValidators";
import { SyntaxValidator, syntaxValidatorBanNode } from "../../syntaxValidation";
import { registerValidationWorkerCallback } from "../../validation";

const validatorConfiguration: FlowReachabilityGraphValidatorConfiguration = {
    steps: {
        order: "sequential",
        steps: [
            {
                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([0, 1]), moveValidatorStartAgent()),
                description: "start printer()"
            },
            {
                order: "parallel",
                steps: [
                    {
                        order: "sequential", // printer()
                        steps: [
                            {
                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([1]), moveValidatorReadPseuCoStruct(0, "message", 42, true)),
                                description: "read 42 from the message field of structure 0 (printer)"
                            },
                            {
                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([1]), moveValidatorPrintLn(/^42$/, false)),
                                description: "print 42 (printer)"
                            },
                            {
                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([1]), moveValidatorReadPseuCoStruct(0, "message", 1337, true)),
                                description: "read 1337 from the message field of structure 0 (printer)"
                            },
                            {
                                moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([1]), moveValidatorPrintLn(/^1337$/, false)),
                                description: "print 1337 (printer)"
                            }
                        ]
                    },
                    {
                        moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([0, 2]), moveValidatorStartAgent()),
                        description: "start sender()"
                    },
                    // sender()
                    {
                        moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([2]), moveValidatorWritePseuCoStruct(0, "message", 42, true)),
                        description: "write 42 to the message field of structure 0 (sender)"
                    },
                    {
                        moveCompletesStep: moveValidatorAnd(moveValidatorParticipatingAgents([2]), moveValidatorWritePseuCoStruct(0, "message", 1337, true)),
                        description: "write 1337 to the message field of structure 0 (sender)"
                    }
                ]
            }
        ]
    },
    isMoveSetAllowed: moveValidatorDisallowDataRaces(),
    restrictedMoves: []
};

const syntaxChecks: SyntaxValidator[] = [
    syntaxValidatorBanNode((node) => node instanceof Struct, "You may not declare a struct."),
    syntaxValidatorBanNode((node) => {
        if (node instanceof VariableDeclarations){
            const type = node.getTypeNode();
            if (type instanceof SimpleTypeNode && type.typeName == "lock") {
                return true;
            }
        } 
        return false;
    }, "You may not explicitly declare a lock.")
];

registerValidationWorkerCallback(flowReachabilityGraphValidator(validatorConfiguration), { mp: false, sm: true }, syntaxChecks);
