import { flowReachabilityGraphValidator, FlowReachabilityGraphValidatorConfiguration } from "../../flowValidation";
import { dropMoveValidatorResultValue, moveValidatorAnd, moveValidatorInitializeMutex, moveValidatorNot, moveValidatorParticipatingAgents, moveValidatorPrintLn, moveValidatorReadPseuCoGlobal, moveValidatorStartAgent, moveValidatorWritePseuCoGlobal } from "../../moveValidators";
import { registerValidationWorkerCallback } from "../../validation";

const validatorConfiguration: FlowReachabilityGraphValidatorConfiguration = {
    steps: {
        order: "sequential",
        steps: [
            {
                moveCompletesStep: moveValidatorStartAgent(),
                description: "start an agent"
            },
            {
                moveCompletesStep: moveValidatorAnd(moveValidatorWritePseuCoGlobal("message", "Hello World!", false), moveValidatorParticipatingAgents([0])),
                description: `write "Hello World!" to the global variable "message" from the main agent`
            },
            {
                moveCompletesStep: moveValidatorAnd(moveValidatorReadPseuCoGlobal("message", "Hello World!", false), moveValidatorParticipatingAgents([1])),
                description: `read "Hello World!" from the global variable "message" from the started agent`
            },
            {
                moveCompletesStep: moveValidatorAnd(moveValidatorPrintLn(/^Hello World!$/, false), moveValidatorParticipatingAgents([1])),
                description: `print "Hello World!" from the started agent`
            }
        ]
    },
    isLegalMove: dropMoveValidatorResultValue(moveValidatorNot(moveValidatorInitializeMutex(), "Your program is attempting to create a mutex. In this exercise, you should not use any shared-memory features besides declaring a global variable.")),
    allowCycleOnMoves: () => ({ success: true, value: null })
};

registerValidationWorkerCallback(flowReachabilityGraphValidator(validatorConfiguration), { mp: false, sm: true });
