import { Move, Token } from "@pseuco/colored-petri-nets";
import { get } from "../../../pseuco-shared-components/util/object";

export const getParticipatingAgents = (move: Move): number[] => {
    const participatingAgents: number[] = [];

    const processTokens = (tokens: Token[]) => tokens.forEach((token) => {
        const id = get(token.color, ["agent", "id"], null);
        if (typeof id === "number" && !participatingAgents.includes(id)) participatingAgents.push(id);
    });

    [...Object.values(move.consumedTokens), ...Object.values(move.producedTokens)].forEach(processTokens);

    return participatingAgents;
};

export const pickRandom = <T>(array: T[]): T => {
    if (!array.length) throw new Error("Cannot pick randomly from an empty array.");
    return array[Math.floor(Math.random() * array.length)];
};
