const path = require('path');
const WorkboxPlugin = require('workbox-webpack-plugin');
const CopyWebpackPlugin = require('copy-webpack-plugin');
const ESLintPlugin = require('eslint-webpack-plugin');

module.exports = (env, args) => ({
    entry: "./src/index.tsx",
    output: {
        path: path.resolve(__dirname, "./dist"),
        filename: "app.js",
        chunkFilename: '[name].bundle.js'
    },
    resolve: {
        extensions: ['.tsx', '.ts', '.jsx', '.js', '.json', '.txt']
    },
    module: {
        rules: [
            {
                test: /\.tsx?$/,
                use: [
                    {
                        loader: "ts-loader"
                    }
                ]
            },
            {
                test: /\.s?css?$/,
                use: [
                    'style-loader', // Creates `style` nodes from JS strings
                    'css-loader', // Translates CSS into CommonJS
                    'sass-loader' // Compiles Sass to CSS
                ]
            },
            {
                test: /\.svg$/,
                loader: 'file-loader'
            },
            {
                test: /\.woff2?$/i,
                type: 'asset/resource',
                dependency: { not: ['url'] },
            },
            {
                test: /\.txt$/,
                loader: 'raw-loader'
            }
        ]
    },
    plugins: [
        new ESLintPlugin(),
        new CopyWebpackPlugin({
            patterns: [
                {from: "./public/*", to: "./[name][ext]"}
            ]
        }),
        args.mode === 'production' ? new WorkboxPlugin.GenerateSW({
            swDest: "./sw.js",
            maximumFileSizeToCacheInBytes: 1024 * 1024 * 50
        }) : new CopyWebpackPlugin({
            patterns: [
                {from: "./hybrid-documents-reader-core/suicide.sw.js", to: "./sw.js"}
            ]
        })
    ],
    devtool: "source-map",
    devServer: {
        hot: true,
        static: './dist',
        client: {
            logging: "info",
            overlay: true,
            progress: true,
            reconnect: true
        }
    },
    performance: {
        hints: false
    }
})
